@@ -373,17 +373,6 @@ search_pager_process (YelpPager *pager)
 {
     d (g_print ("search_pager_process\n"));
 
-#ifdef ENABLE_BEAGLE
-    if (beagle_client == NULL) {
-	GError *error = NULL;
-	g_set_error (&error, YELP_ERROR, YELP_ERROR_PROC,
-		     _("Your search could not be processed. There "
-		       "is no connection to the beagle daemon."));
-	yelp_pager_error (YELP_PAGER (pager), error);
-	return FALSE;
-    }
-#endif /* ENABLE_BEAGLE */
-
     yelp_pager_set_state (pager, YELP_PAGER_STATE_PARSING);
     g_signal_emit_by_name (pager, "parse");
 
@@ -520,12 +509,9 @@ compare_hits (gconstpointer  a,
     const char *langa, *langb;
     gboolean a_preferred = TRUE, b_preferred = TRUE;
 
-    langa = beagle_hit_get_property (*hita, "fixme:language");
-    langb = beagle_hit_get_property (*hitb, "fixme:language");
-
-    if (langa)
+    if (beagle_hit_get_one_property (*hita, "fixme:language", &langa))
 	a_preferred = check_lang(langa);
-    if (langb)
+    if (beagle_hit_get_one_property (*hitb, "fixme:language", &langb))
 	b_preferred = check_lang(langb);
 
     if (a_preferred != b_preferred) {
@@ -562,7 +548,7 @@ finished_cb (BeagleQuery            *query,
     for (i = 0; i < 10 && i < priv->hits->len; i++) {
 	BeagleHit *hit = g_ptr_array_index (priv->hits, i);
 	xmlNode *child;
-	static float score_fake = 0;
+	/* static float score_fake = 0; */
 	char *score;
 	const char *property;
 	BeagleSnippetRequest *request;
@@ -572,11 +558,9 @@ finished_cb (BeagleQuery            *query,
 	xmlSetProp (child, BAD_CAST "uri", BAD_CAST beagle_hit_get_uri (hit));
 	xmlSetProp (child, BAD_CAST "parent_uri", 
 		    BAD_CAST beagle_hit_get_parent_uri (hit));
-	property = beagle_hit_get_property (hit, "dc:title");
-	if (property)
+	if (beagle_hit_get_one_property (hit, "dc:title", &property))
 	    xmlSetProp (child, BAD_CAST "title", BAD_CAST property);
-	property = beagle_hit_get_property (hit, "fixme:base_title");
-	if (property)
+	if (beagle_hit_get_one_property (hit, "fixme:base_title", &property))
 	    xmlSetProp (child, BAD_CAST "base_title", BAD_CAST property);
 
 	score = g_strdup_printf ("%f", beagle_hit_get_score (hit));
@@ -630,39 +614,42 @@ search_pager_process_idle (YelpSearchPager *pager)
     xmlDocSetRootElement (priv->search_doc, priv->root);
 
 #ifdef ENABLE_BEAGLE
-    query = beagle_query_new ();
+    if (beagle_client != NULL) {
+	query = beagle_query_new ();
 
-    beagle_query_set_max_hits (query, 10000);
-    beagle_query_add_text (query, priv->search_terms);
-    beagle_query_add_mime_type (query, "text/html");
-    beagle_query_add_mime_type (query, "application/xhtml+xml");
-    beagle_query_add_mime_type (query, "text/xml");
-    beagle_query_add_mime_type (query, "application/xml");
-    beagle_query_add_mime_type (query, "application/docbook+xml");
-    beagle_query_add_mime_type (query, "text/x-docbook-entry");
+	beagle_query_set_max_hits (query, 10000);
+	beagle_query_add_text (query, priv->search_terms);
+	beagle_query_add_source (query, "documentation");
 
-    priv->hits = g_ptr_array_new ();
+	priv->hits = g_ptr_array_new ();
 
-    g_signal_connect (query, "hits-added",
-		      G_CALLBACK (hits_added_cb),
-		      pager);
+	g_signal_connect (query, "hits-added",
+			  G_CALLBACK (hits_added_cb),
+			  pager);
     
-    g_signal_connect (query, "finished",
-		      G_CALLBACK (finished_cb),
-		      pager);
+	g_signal_connect (query, "finished",
+			  G_CALLBACK (finished_cb),
+			  pager);
 	
-    beagle_client_send_request_async (beagle_client, BEAGLE_REQUEST (query), &error);
+	beagle_client_send_request_async (beagle_client, BEAGLE_REQUEST (query), &error);
 
-    if (error) {
-	d(g_print ("error: %s\n", error->message));
+	if (error) {
+	    d(g_print ("error: %s\n", error->message));
+	}
+
+	g_clear_error (&error);
+    } else {
+	g_warning ("beagled not running, using basic search support.");
     }
-
-    g_clear_error (&error);
 #endif /* ENABLE_BEAGLE */
 
-#ifndef ENABLE_BEAGLE
-    gtk_idle_add ((GtkFunction) slow_search_setup,
-		  pager);
+#ifdef ENABLE_BEAGLE
+    if (beagle_client == NULL) {
+#endif
+	gtk_idle_add ((GtkFunction) slow_search_setup,
+		      pager);
+#ifdef ENABLE_BEAGLE
+    }
 #endif
 
     return FALSE;
@@ -1193,6 +1180,13 @@ slow_search_process (YelpSearchPager *pager)
 
     pending_searches = g_slist_remove_link (pending_searches, first);
 
+    if (first == NULL) {
+	gtk_idle_add_priority (G_PRIORITY_LOW,
+			       (GtkFunction) process_xslt,
+			       pager);
+	return FALSE;
+    }
+
     c = (SearchContainer *) first->data;
 
     xmlSAXUserParseFile (&handlers, c, c->base_filename);
@@ -1209,7 +1203,6 @@ slow_search_process (YelpSearchPager *pager)
 	    break;
 	}
     }
-
     /* Cleanup the container and delete it */
     g_free (c->current_subsection);
     g_free (c->result_subsection);
@@ -1219,7 +1212,6 @@ slow_search_process (YelpSearchPager *pager)
     g_free (c->snippet);
     g_hash_table_destroy (c->entities);
 
-
     if (pending_searches) {
 	g_strfreev (c->search_term);
 	g_free (c);
@@ -1416,11 +1408,15 @@ process_man_result (YelpSearchPager *pager, gchar *result, gchar **terms)
 void
 process_info_result (YelpSearchPager *pager, gchar *result, gchar **terms)
 {
-    gchar ** split = g_strsplit (result, "\n", -1);
+    gchar ** split = NULL;
     gint i;
 
+    split = g_strsplit (result, "\n", -1);
+    if (split == NULL)
+	return;
+
     for (i=0;split[i];i++) {
-	gchar ** line = g_strsplit (split[i], "--", 3); 
+	gchar ** line = NULL;
 	gchar *filename = NULL;
 	gchar *desc = NULL;
 	gchar *title = NULL;
@@ -1428,7 +1424,8 @@ process_info_result (YelpSearchPager *pager, gchar *result, gchar **terms)
 	gchar *tmp;
 	gchar *tmp1;
 	gchar *file_name;
-
+	
+	line = g_strsplit (split[i], "--", 3); 
 	if (g_strv_length (line) != 2) {
 	    g_strfreev (line);
 	    continue;
@@ -1441,6 +1438,11 @@ process_info_result (YelpSearchPager *pager, gchar *result, gchar **terms)
 	tmp = g_strdup (g_strchomp (line[0]));
 	tmp++;
 	tmp1 = strstr (tmp, "\"");
+	if (!tmp1) {
+	    g_strfreev (line);
+	    g_free (tmp);
+	    continue;
+	}	    
 	file_name = g_strndup (tmp, tmp1-tmp);
 	tmp++;
 	tmp1 = strstr (tmp, ")");
@@ -1481,19 +1483,23 @@ search_process_man (YelpSearchPager *pager, gchar **terms)
     gint i=0;
     for (i=0; terms[i]; i++) {
 	gchar *command;
-	gchar *stdout_str;
+	gchar *stdout_str = NULL;
 	gint exit_code;
-	command = g_strconcat("apropos ", terms[i], NULL);
-    
+	gchar *tmp = NULL;
+
+	tmp = g_strescape (terms[i], NULL);
+	tmp = g_strdelimit (tmp, "\'", '\'');
+	command = g_strconcat("apropos ", tmp, NULL);
+
 	if (g_spawn_command_line_sync (command, &stdout_str, NULL, 
 				       &exit_code, NULL) && exit_code == 0) {
 	    process_man_result (pager, stdout_str, terms);
 
 	}
+	g_free (tmp);
 	g_free (stdout_str);
 	g_free (command);
     }
-
     return;
 }
 
@@ -1503,15 +1509,21 @@ search_process_info (YelpSearchPager *pager, gchar **terms)
     gint i=0;
     for (i=0; terms[i]; i++) {
 	gchar *command;
-	gchar *stdout_str;
-	gchar *stderr_str;
+	gchar *stdout_str = NULL;
+	gchar *stderr_str = NULL;
+	gchar *tmp;
 	gint exit_code;
-	command = g_strconcat("info --apropos ", terms[i], NULL);
+	
+	tmp = g_strescape (terms[i], NULL);
+	tmp = g_strdelimit (tmp, "\'", '\'');
+	command = g_strconcat("info --apropos ", tmp, NULL);
     
 	if (g_spawn_command_line_sync (command, &stdout_str, &stderr_str, 
-				       &exit_code, NULL) && exit_code == 0) {
+				       &exit_code, NULL) && 
+	    stdout_str != NULL) {
 	    process_info_result (pager, stdout_str, terms);	    
 	}
+	g_free (tmp);
 	g_free (stdout_str);
 	g_free (stderr_str);
 	g_free (command);
